// Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
// This file is part of Tasty Safari.
//
// Tasty Safari is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Tasty Safari is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.tastysafari.swingui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.*;
import javax.swing.*;
import org.w3c.dom.Document;

import com.apple.eawt.*;

import org.ninjasoft.tastysafari.logic.*;
import org.ninjasoft.tastysafari.data.PostEntry;

public class MainWindow extends JFrame implements StatusListener, Runnable {
    public static final String VERSION = "v1.1";
    private Actions actions = new Actions();
    private JTextField urlText = new JTextField("del.icio.us");
    private JTextField usernameText = new JTextField("");
    private JTextField passwordText = new JPasswordField();
    private JButton okButton = new JButton("Proceed");
    private JButton cancelButton = new JButton("Cancel");
    private JEditorPane stepText = new JEditorPane("text/html", "");
    int currentStep = -1;
    private static String[] STEP_DESCRIPTIONS = new String[] {
            "Read existing Safari bookmark file",
            "Download bookmarks",
            "Process data",
            "Backup existing Safari bookmark file",
            "Write new Safari bookmark file"};
    public String[] stepDetails = new String[5];
    public SafariBookmarkEditor bookmarkEditor = null;
    
    public MainWindow() {
        super();
        bookmarkEditor = new SafariBookmarkEditor();
        resetStepDetails();
        initializeGui();
        initializeEvents();
        initializeApple();
        urlText.setEditable(false);
        usernameText.setSelectionStart(0);
        usernameText.setSelectionEnd(usernameText.getText().length());
        usernameText.grabFocus();
    }
    
    public void initializeApple() {
        System.setProperty("com.apple.mrj.application.apple.menu.about.name", "Tasty Safari");
        System.setProperty("com.apple.laf.useScreenMenuBar", "true");
        System.setProperty("apple.awt.brushMetalLook", "true");
        Application application = Application.getApplication();
        application.addApplicationListener(new ApplicationAdapter() {
            public void handleQuit(ApplicationEvent event) {
                System.exit(0);
            }
        });
        application.setEnabledAboutMenu(false);
        application.setEnabledPreferencesMenu(false);
        
        
    }
    
    public void resetStepDetails() {
        currentStep = -1;
        stepDetails[0] = bookmarkEditor.getBookmarkFilename();
        stepDetails[1] = "0 item(s) downloaded";
        stepDetails[2] = "";
        stepDetails[3] = "There are currently " + bookmarkEditor.getBackupCount() + " previous backup(s)";
        stepDetails[4] = "0 bytes written";
        stepText.setText(buildStatusString(currentStep));
    }
    
    private void initializeGui() {
        this.setTitle("Tasty Safari " + VERSION);
        this.setSize(400, 350);
        stepText.setOpaque(false);
        Dimension windowSize = this.getSize();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        this.setLocation(screenSize.width/2 - windowSize.width/2, screenSize.height/2 - windowSize.height/2);
        this.setResizable(false);
        Container pane = this.getContentPane();
        pane.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        // Server line
        c.gridx = 0;
        c.gridy = 0;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.EAST;
        c.weightx = 0;
        c.weighty = 0;
        c.gridwidth = 1;
        c.insets = new Insets(2, 15, 2, 2);
        pane.add(new Label("Server"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1;
        c.weighty = 0;
        c.gridwidth = 3;
        c.insets = new Insets(2, 2, 2, 15);
        pane.add(urlText, c);
        // Username line
        c.gridx = 0;
        c.gridy++;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.EAST;
        c.weightx = 0;
        c.weighty = 0;
        c.gridwidth = 1;
        c.insets = new Insets(2, 15, 2, 2);
        pane.add(new Label("Username"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1;
        c.weighty = 0;
        c.gridwidth = 3;
        c.insets = new Insets(2, 2, 2, 15);
        pane.add(usernameText, c);
        // Password line
        c.gridx = 0;
        c.gridy++;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.EAST;
        c.weightx = 0;
        c.weighty = 0;
        c.gridwidth = 1;
        c.insets = new Insets(2, 15, 2, 2);
        pane.add(new Label("Password"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1;
        c.weighty = 0;
        c.gridwidth = 3;
        c.insets = new Insets(2, 2, 2, 15);
        pane.add(passwordText, c);
        
        //Spacer
        c.gridx = 0;
        c.gridy++;
        c.fill = GridBagConstraints.BOTH;
        c.anchor = GridBagConstraints.CENTER;
        c.weightx = 2;
        c.weighty = 2;
        c.gridwidth = 4;
        c.insets = new Insets(2, 15, 2, 15);
        pane.add(stepText, c);
        // Buttons
        c.gridx = 0;
        c.gridy++;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 0;
        c.weighty = 0;
        c.gridwidth = 1;
        c.insets = new Insets(15, 15, 15, 15);
        pane.add(cancelButton, c);
        c.gridx += 2;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.EAST;
        c.weightx = 1;
        c.weighty = 0;
        c.gridwidth = 2;
        c.insets = new Insets(15, 15, 15, 15);
        pane.add(okButton, c);
        
    }
    
    private void initializeEvents() {
        this.setDefaultCloseOperation(JDialog.EXIT_ON_CLOSE);
        cancelButton.setEnabled(false);
        okButton.setActionCommand("okButton");
        okButton.addActionListener(actions);
        cancelButton.setActionCommand("cancelButton");
        cancelButton.addActionListener(actions);
    }
    
    public void setStatus(String s) {
        if ((s != null) && (s.length() > 0))
                stepDetails[2] = s;
        stepText.setText(buildStatusString(currentStep));
    }
    
    private String buildStatusString(int step) {
        StringBuffer result = new StringBuffer();
        result.append("<html><head></head><body>");
        result.append("<b><u>Steps to be performed</u></b><br>");
        //result.append("<ul>");
        for (int i=0; i<STEP_DESCRIPTIONS.length; i++) {
            result.append("&nbsp;&nbsp;&nbsp;");
            if (step == i)
                result.append("<b>");
            result.append("&raquo;&nbsp;");
            result.append(STEP_DESCRIPTIONS[i]);
            if (step == i)
            	result.append("</b>");
            result.append("<br>");
            result.append("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
            result.append(stepDetails[i]);
            result.append("<br>");
            //result.append("</li>");
        }
        //result.append("</ul>");
        result.append("</body></html>");
        return result.toString();
    }
    
    private void startThread() {
        enableControls(false);
        Thread childThread = new Thread(this);
        childThread.start();
    }
    
    private void stopThread() {
        //TODO:
    }
    
    private void enableControls(boolean enabled) {
        this.okButton.setEnabled(enabled);
        this.cancelButton.setEnabled(!enabled);
        this.usernameText.setEnabled(enabled);
        this.passwordText.setEnabled(enabled);
    }
    

    public void run() {
        String state = "initializing";
        int count = 0;
        Vector entries = new Vector();
        Document doc = null;
        resetStepDetails();
        try{
            // #####
            this.currentStep = 0;
            setStatus("");
            doc = this.bookmarkEditor.load();
            // #####
            this.currentStep++;
            setStatus("");
            state = "constructing API object";
            DeliciousApi delicious = new DeliciousApi(usernameText.getText(), passwordText.getText(), urlText.getText());
            state = "downloading dates";
            String[] dates = delicious.getInboxDates();
            stepDetails[currentStep] = "downloaded " + dates.length + " date(s)";
            setStatus("");
            for (int i=0; i<dates.length; i++) {
                state = "downloading posts for date #" + i;
                PostEntry[] posts = delicious.getInbox(dates[i]);
                entries.add(posts);
                count += posts.length;
            }
            stepDetails[currentStep] = stepDetails[currentStep] + ", " + count + " total posts";
            // #####
            this.currentStep++;
            setStatus("");
            SafariBookmarkMerger merger = new SafariBookmarkMerger();
            merger.setUsername(usernameText.getText());
            merger.merge(doc, entries, this);
            // #####
            this.currentStep++;
            setStatus("");
            bookmarkEditor.backupBookmarks();
            stepDetails[currentStep] = "There are currently " + bookmarkEditor.getBackupCount() + " previous backup(s)";
            // #####
            this.currentStep++;
            setStatus("");
            bookmarkEditor.save(doc);
            File f = new File(bookmarkEditor.getBookmarkFilename());
            stepDetails[currentStep] = "" + f.length() + " bytes written";
            this.currentStep = -1;
            setStatus("");
        } catch(Throwable t) {
            t.printStackTrace();
            JOptionPane.showMessageDialog(this, "Problem " + state + ":\n" + t.toString(), "Unexpected Error", JOptionPane.ERROR_MESSAGE);
        }finally{
            enableControls(true);
            this.currentStep = -1;
            setStatus("");
        }
    }
    
    public class Actions implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			String source = event.getActionCommand();
            source = (source == null) ? "" : source;
            if (source.equals("okButton"))
                startThread();
            if (source.equals("cancelButton"))
                stopThread();
		}
    }
    
    public static void main(String[] argv) {
        MainWindow window = new MainWindow();
        window.setVisible(true);
    }
}
